unit benchmark;

interface
uses classes,windows,addon,sysutils,modulelist,forms,comctrls,Contnrs,grids,ValEdit,dglopengl,graphics,ExtCtrls;

type

TBenchmarkProperty=class
 Name,Value:string;
 next:TBenchmarkProperty;
 list:string;
 image:TBitmap;
destructor destroy;override;
end;

TBenchmark=class(TInterfacedObject,IBenchmark)
public
 fpropertytree:TTreeView;
 fnodelist:TObjectList;
 fPropertyList:TValueListEditor;
 fResultTree:TTreeView;
 fResultList:TListView;
 fResultImage:TImage;
 fmodulelist:TModuleList;
 fbform:TForm;
 fabort:boolean;
 fdc:hdc;
 frc:HGLRC;
 function CreatePath(tv:TTreeView;n:TTreeNode;const path:string):TTreeNode;
 procedure InitForm;
 procedure CloseForm;
public
 property PropertyTree:TTreeView read fpropertytree write fpropertytree;
 property PropertyList:TValueListEditor read fPropertyList write fPropertyList;
 property ResultTree:TTreeView read fResultTree write fResultTree;
 property ResultList:TListView read fResultList write fResultList;
 property ResultImage:TImage read fResultImage write fResultImage;
 procedure PropertyTreeChange(sender:TObject;Node: TTreeNode);
 procedure ResultTreeChange(sender:TObject;Node: TTreeNode);
 procedure PropertyListValidate(Sender: TObject;ACol,ARow: Integer; const KeyName, KeyValue: String);
 function AddProperty(propname,defvalue,list:pchar):integer;
 function GetProperty(index:integer;val:pchar):integer;
 function AddResult(propname,val:pchar):integer;
 procedure AddResultImage(propname:pchar;bitmap:HBITMAP);
 procedure OnKeyPress(Sender: TObject; var Key: Char);
 procedure Run;
 function ProcessMessages:boolean;
 procedure LoadModules;
 procedure FreeModules;
 constructor create;
 destructor destroy;override;
end;

implementation
uses unit1;

// TBenchmarkProperty

destructor TBenchmarkProperty.destroy;
begin
 image.Free;
 inherited;
end;

// TBenchmark

function TBenchmark.CreatePath(tv:TTreeView;n:TTreeNode;const path:string):TTreeNode;
var
 i,p:integer;
 n2:TTreeNode;
 path1,path2:string;
begin
if path='' then
begin
 result:=n;
 exit;
end;
n2:=nil;
p:=pos('\',path);
if p=0 then
begin
 path1:=path;
 path2:='';
end
else
begin
 path1:=copy(path,1,p-1);
 path2:=copy(path,p+1,length(path)-p);
end;

if n=nil then
begin
for i:=0 to tv.items.Count-1 do
 if (tv.Items[i].Level=0) and SameText(tv.Items[i].Text,path1) then
 begin
  n2:=tv.Items[i];
  break;
 end;
end
else
begin
for i:=0 to n.Count-1 do
 if SameText(n.item[i].Text,path1) then
 begin
  n2:=n.item[i];
  break;
 end;
end;
if n2=nil then n2:=tv.Items.AddChild(n,path1);
result:=CreatePath(tv,n2,path2);
end;

function TBenchmark.AddProperty(propname,defvalue,list:pchar):integer;
var
 n:TTreeNode;
 p:TBenchmarkProperty;
 pname:string;
begin
n:=CreatePath(PropertyTree,nil,ExtractFilePath(propname));
pname:=ExtractFileName(propname);
p:=n.Data;
while p<>nil do
begin
 if sametext(p.Name,pname) then break;
 p:=p.next;
end;
if p=nil then
begin
 p:=TBenchmarkProperty.Create;
 p.Name:=ExtractFileName(propname);
 p.Value:=defvalue;
 p.list:=list;
 p.next:=n.data;
 fnodelist.Add(p);
 n.Data:=p;
end;
result:=integer(p);
end;

function TBenchmark.GetProperty(index:integer;val:pchar):integer;
begin
StrPCopy(val,TBenchmarkProperty(index).Value);
result:=length(TBenchmarkProperty(index).Value);
end;

function TBenchmark.AddResult(propname,val:pchar):integer;
var
 n:TTreeNode;
 p:TBenchmarkProperty;
 pname:string;
begin
n:=CreatePath(ResultTree,nil,ExtractFilePath(StringReplace(propname,'/','\',[rfreplaceall])));
pname:=ExtractFileName(propname);
p:=n.Data;
while p<>nil do
begin
 if sametext(p.Name,pname) then break;
 p:=p.next;
end;
if p=nil then
begin
 p:=TBenchmarkProperty.Create;
 p.Name:=ExtractFileName(propname);
 p.Value:=val;
 p.next:=n.data;
 fnodelist.Add(p);
 n.Data:=p;
end;
result:=integer(p);
end;

procedure TBenchmark.AddResultImage(propname:pchar;bitmap:HBITMAP);
var
 p:TBenchmarkProperty;
begin
p:=TBenchmarkProperty(AddResult(propname,'_Default'));
p.image:=TBitmap.Create;
p.image.Handle:=bitmap;
end;

procedure TBenchmark.PropertyTreeChange(sender:TObject;Node: TTreeNode);
var
 r:integer;
 p:TBenchmarkProperty;
begin
if Node<>nil then
begin
 PropertyList.Strings.Clear;
 p:=Node.data;
 while p<>nil do
 begin
  r:=PropertyList.InsertRow(p.Name,p.Value,true);
  PropertyList.Strings.Objects[r-1]:=p;
  if p.list<>'' then
  begin
   PropertyList.ItemProps[r-1].ReadOnly:=true;
   with PropertyList.ItemProps[r-1] do
   begin
    EditStyle:=esPickList;
    PickList.Text:=StringReplace(p.list,'|',#13#10,[rfreplaceall]);
   end;
  end;

  p:=p.next;
 end;
end
else
  PropertyList.Strings.Clear;
end;

procedure TBenchmark.ResultTreeChange(sender:TObject;Node:TTreeNode);
var
 p:TBenchmarkProperty;
begin
if Node<>nil then
begin
 ResultList.Items.Clear;
 p:=Node.data;
 if (p<>nil) and (p.image<>nil) then
 begin
  ResultImage.Parent.Show;
  ResultImage.Picture.Bitmap:=p.image;
 end
 else
  ResultImage.Parent.Hide;
 while p<>nil do
 begin
  with ResultList.Items.Add do
  begin
   Caption:=p.Name;
   SubItems.Add(p.Value);
  end;
  p:=p.next;
 end;
end
else
 ResultList.Items.Clear;
end;

procedure TBenchmark.PropertyListValidate(Sender: TObject;ACol,ARow: Integer; const KeyName, KeyValue: String);
begin
TBenchmarkProperty(PropertyList.Strings.Objects[ARow-1]).Value:=KeyValue;
end;

function TBenchmark.ProcessMessages:boolean;
begin
Application.ProcessMessages;
result:=not fabort;
end;

procedure TBenchmark.LoadModules;
var
 r:TSearchRec;
 i:integer;
begin
i:=FindFirst(ExtractFilePath(Application.ExeName)+'*.dll',faAnyFile,r);
while i=0 do
begin
 fmodulelist.LoadModule(ExtractFilePath(Application.ExeName)+r.Name);
 i:=FindNext(r);
end;
FindClose(r);
for i:=0 to fmodulelist.List.Count-1 do
 IBenchmarkModule(fmodulelist.List[i]).Init(self);
end;

procedure TBenchmark.FreeModules;
begin
fmodulelist.List.Clear;
end;

procedure TBenchmark.OnKeyPress(Sender: TObject; var Key: Char);
begin
fabort:=key=#27;
end;

procedure TBenchmark.InitForm;
begin
fbform:=TForm.Create(form1);
fbform.Left:=0;
fbform.Top:=0;
fbform.BorderStyle:=bsNone;
fbform.Width:=Screen.Width;
fbform.Height:=Screen.Height;
fbform.Show;
fbform.OnKeyPress:=OnKeyPress;
fdc:=GetDC(fbform.Handle);
InitOpenGL;
frc:=CreateRenderingContext(fdc,[opDoubleBuffered],32,24,8,0,0,0);
ActivateRenderingContext(fdc,frc);
glClear(GL_COLOR_BUFFER_BIT);
SwapBuffers(fdc);
end;

procedure TBenchmark.CloseForm;
begin
DeactivateRenderingContext;
DestroyRenderingContext(frc);
ReleaseDC(fbform.Handle,fdc);
fbform.free;
end;


procedure TBenchmark.Run;
var
 i:integer;
begin
fabort:=false;
InitForm;
try
for i:=0 to fmodulelist.List.count-1 do
begin
 IBenchmarkmodule(fmodulelist.List[i]).Run;
 if fabort then break;
end;
finally
 CloseForm;
end;
end;

constructor TBenchmark.create;
begin
inherited;
fnodelist:=TObjectList.Create;
fmodulelist:=TModuleList.create;
end;

destructor TBenchmark.destroy;
begin
fnodelist.Free;
fmodulelist.free;
inherited;
end;


end.
