unit module;

interface
uses classes,windows,addon,dglopengl,graphics;

type

TBaseModuleClass=class of TBaseModule;

TBaseModule=class(TInterfacedObject,IBenchmarkModule)
private
 fname,fcategory,finfotext:string;
 freq,timer,timerstart:int64;
protected
 fbenchmark:IBenchmark;
 framecount:integer;
 time,fps:single;
 fabort:boolean;
public
 function GetCategory:PChar;
 function GetInfoText:PChar;
 function GetName:PChar;
 function Run:Boolean;virtual;
 procedure DrawFrame;virtual;
 procedure ListResults;virtual;
 procedure RegisterProperties;virtual;
 procedure InitRessources;virtual;
 procedure FreeRessources;virtual;
 function AddProperty(const propname,defvalue,list:string):integer;
 function GetProperty(const index:integer):string;
 procedure AddResult(const propname,val:string);
 procedure AddResultImage(propname:pchar;bitmap:TBitmap);
 function CreateScreenshot:TBitmap;
 procedure CopyScreenShot(const path:string);
 procedure Init(const bm:IBenchmark);
 constructor create;virtual;
end;

procedure RegisterModule(m:TBaseModuleClass);
function GetModuleCount:integer;
function CreateModule(index:integer):IBenchmarkModule;

implementation

var
 modules:TList;

procedure RegisterModule(m:TBaseModuleClass);
begin
 modules.Add(m);
end;

function GetModuleCount:integer;
begin
 result:=modules.Count;
end;

function CreateModule(index:integer):IBenchmarkModule;
begin
 result:=TBaseModuleClass(modules[index]).Create;
end;

function TBaseModule.GetCategory:PChar;
begin
 result:=pchar(fcategory);
end;

function TBaseModule.GetInfoText:PChar;
begin
 result:=pchar(finfotext);
end;

function TBaseModule.GetName:PChar;
begin
 result:=pchar(fname);
end;

procedure TBaseModule.DrawFrame;
begin
end;

function TBaseModule.Run:Boolean;
var
 dc:hdc;
begin
dc:=wglGetCurrentDC;
InitRessources;
QueryPerformanceCounter(timerstart);
time:=0;
framecount:=0;
fabort:=false;
try
 while fbenchmark.ProcessMessages and not fabort do
 begin
  DrawFrame;
  SwapBuffers(dc);
  QueryPerformanceCounter(timer);
  time:=(timer-timerstart)/freq;
  inc(framecount);
  fps:=framecount/time;
 end;
finally
 FreeRessources;
end;
ListResults;
result:=fbenchmark.ProcessMessages;
end;

procedure TBaseModule.ListResults;
begin

end;

procedure TBaseModule.RegisterProperties;
begin
end;

procedure TBaseModule.InitRessources;
begin
end;

procedure TBaseModule.FreeRessources;
begin
end;

function TBaseModule.AddProperty(const propname,defvalue,list:string):integer;
begin
result:=fbenchmark.AddProperty(pchar(propname),pchar(defvalue),pchar(list));
end;

function TBaseModule.GetProperty(const index:integer):string;
var
 buffer:array[0..255] of char;
begin
fbenchmark.GetProperty(index,@buffer[0]);
result:=buffer;
end;

procedure TBaseModule.AddResult(const propname,val:string);
begin
fbenchmark.AddResult(pchar(propname),pchar(val));
end;

procedure TBaseModule.AddResultImage(propname:pchar;bitmap:TBitmap);
begin
fbenchmark.AddResultImage(propname,bitmap.handle);
bitmap.ReleaseHandle;
end;

function TBaseModule.CreateScreenshot:TBitmap;
var
 viewp:array[0..3] of integer;
 i:integer;
begin
result:=TBitmap.Create;
glGetIntegerv(GL_VIEWPORT,@viewp);
result.Width:=viewp[2]-viewp[0];
result.height:=viewp[3]-viewp[1];
result.PixelFormat:=pf24bit;
for i:=0 to result.height-1 do
 glReadPixels(viewp[0],viewp[1]+i,viewp[2],1,GL_BGR,GL_UNSIGNED_BYTE,result.ScanLine[result.height-1-i]);
end;

procedure TBaseModule.CopyScreenShot(const path:string);
var
  screenshot:TBitmap;
begin
screenshot:=CreateScreenshot;
AddResultImage(pchar(path),screenshot);
screenshot.Free;
end;

procedure TBaseModule.Init(const bm:IBenchmark);
begin
InitOpenGL;
fbenchmark:=bm;
RegisterProperties;
QueryPerformanceFrequency(freq);
end;

constructor TBaseModule.create;
begin
inherited;
end;

exports
 GetModuleCount,
 CreateModule;

initialization
begin
 modules:=TList.Create;
end;

finalization
begin
 modules.free;
end;


end.
